@@ -17,7 +17,7 @@
 from __future__ import annotations
 from abc import ABC, abstractmethod
 
-import numpy as np
+import numpy.typing as npt
 from typing import Any, Generic, Optional, Sequence, Union
 from uuid import UUID, uuid4
 
@@ -27,15 +27,14 @@ from .base import Instance
 
 
 class TextInstance(Instance[KT, str, VT, str], ABC, Generic[KT, VT]):
-    
     @property
     @abstractmethod
-    def map_to_original(self) -> Optional[np.ndarray]:
+    def map_to_original(self) -> Optional[npt.NDArray[Any]]:
         raise NotImplementedError
 
     @map_to_original.setter
     @abstractmethod
-    def map_to_original(self, value: Optional[np.ndarray]) -> None:
+    def map_to_original(self, value: Optional[npt.NDArray[Any]]) -> None:
         raise NotImplementedError
 
     @property
@@ -47,28 +46,33 @@ class TextInstance(Instance[KT, str, VT, str], ABC, Generic[KT, VT]):
     @abstractmethod
     def split_marker(self, value: Any) -> None:
         raise NotImplementedError
-    
+
     @property
     @abstractmethod
     def tokenized(self) -> Optional[Sequence[str]]:
         raise NotImplementedError
-    
+
     @tokenized.setter
     @abstractmethod
     def tokenized(self, value: Sequence[str]) -> None:
         raise NotImplementedError
 
 
-class MemoryTextInstance(DataPoint[Union[KT, UUID], str, VT, str], TextInstance[Union[KT, UUID], VT], Generic[KT, VT]):
-   
-    def __init__(self, 
-                 identifier: Union[KT, UUID], 
-                 data: str, 
-                 vector: Optional[VT], 
-                 representation: Optional[str] = None, 
-                 tokenized: Optional[Sequence[str]] = None,
-                 map_to_original: Optional[np.ndarray] = None,
-                 split_marker: Optional[Any] = None) -> None:
+class MemoryTextInstance(
+    DataPoint[Union[KT, UUID], str, VT, str],
+    TextInstance[Union[KT, UUID], VT],
+    Generic[KT, VT],
+):
+    def __init__(
+        self,
+        identifier: Union[KT, UUID],
+        data: str,
+        vector: Optional[VT],
+        representation: Optional[str] = None,
+        tokenized: Optional[Sequence[str]] = None,
+        map_to_original: Optional[npt.NDArray[Any]] = None,
+        split_marker: Optional[Any] = None,
+    ) -> None:
         representation = data if representation is None else representation
         super().__init__(identifier, data, vector, representation)
         self._tokenized = tokenized
@@ -76,33 +80,36 @@ class MemoryTextInstance(DataPoint[Union[KT, UUID], str, VT, str], TextInstance[
         self._split_marker = split_marker
 
     @property
-    def map_to_original(self) -> Optional[np.ndarray]:
+    def map_to_original(self) -> Optional[npt.NDArray[Any]]:
         return self._map_to_original
 
     @map_to_original.setter
-    def map_to_original(self, value: Optional[np.ndarray]) -> None:
+    def map_to_original(self, value: Optional[npt.NDArray[Any]]) -> None:
         self._map_to_original = value
 
     @property
     def split_marker(self) -> Optional[Any]:
-        return self._split_marker 
+        return self._split_marker
 
     @split_marker.setter
     def split_marker(self, value: Any):
         self._split_marker = value
-    
+
     @property
     def tokenized(self) -> Optional[Sequence[str]]:
         return self._tokenized
-    
+
     @tokenized.setter
     def tokenized(self, value: Sequence[str]) -> None:
         self._tokenized = value
 
-        
-
-class TextInstanceProvider(AbstractMemoryProvider[MemoryTextInstance[KT, VT], Union[KT, UUID], str, VT, str], Generic[KT, VT]):
 
+class TextInstanceProvider(
+    AbstractMemoryProvider[
+        MemoryTextInstance[KT, VT], Union[KT, UUID], str, VT, str
+    ],
+    Generic[KT, VT],
+):
     def create(self, *args: Any, **kwargs: Any):
         new_key = uuid4()
         new_instance = MemoryTextInstance[KT, VT](new_key, *args, **kwargs)
